# example3.py
#
# This example selects some information from the "pubs2" database.  It
# illustrates binding of both aggregate and compute results.
#
# Note that this example will only work if the "pubs2" database exists
# on your SQL Server. Consult the Installation Guide for information
# about installing the "pubs2" database.
import sys
from mssqldb import *

import exutils

PLEN = 4 
DATEPRINT = 26
MONEYPRINT = 12

print "Demo of SQL queries in a command batch\n"

# Initialize DB-Library.
if not dbinit():
    sys.exit(1)

# Install the user-supplied error-handling and message-handling
# routines. They are defined at the bottom of this source file.
dberrhandle(exutils.err_handler)
dbmsghandle(exutils.msg_handler)

# Set up the login information.
login = dblogin()
login.dbsetluser(exutils.USER)
login.dbsetlpwd(exutils.PASSWORD)
login.dbsetlapp("example3")

dbproc = login.dbopen(exutils.SERVER)

# Send a "use database" command.
dbproc.dbuse("pubs2")

# Put the SQL statement into the command buffer.
dbproc.dbcmd("select pub_id, pubdate, price, avg(price), total_sales,")
dbproc.dbcmd(" avg(total_sales), sum(total_sales) from titles")
dbproc.dbcmd(" group by pub_id")
dbproc.dbcmd(" order by pub_id")
dbproc.dbcmd(" compute count(pub_id) by pub_id")

# Send the command buffer to SQL Server for execution.
dbproc.dbsqlexec()

# Using the aggregates "sum" and "avg" with the COMPUTE clause
# necessitates special handling when binding the results. Since each
# aggregate creates a new column, this is accounted for in the bind.
# Notice that avg(price) is the fourth column in the select-list, and
# is also specified as the fourth column in the dbbind() routine.
#
# The COMPUTE clause creates a compute row, which requires a special
# bind routine called dbaltbind().

# Declare the datatypes for the columns in the table "titles".
pcount = DataBuf(SQLINT4)
sales = DataBuf(SQLINT4)
salesavg = DataBuf(SQLINT4)
sumsale = DataBuf(SQLINT4)
date = DataBuf(SQLCHAR, DATEPRINT)
price = DataBuf(SQLCHAR, MONEYPRINT)
priceavg = DataBuf(SQLCHAR, MONEYPRINT)
pubid = DataBuf(SQLCHAR, PLEN)

while 1:
    result_code = dbproc.dbresults()
    if result_code == NO_MORE_RESULTS:
        break
    if result_code == SUCCEED:
        dbproc.dbbind(1, pubid, 0)
        dbproc.dbbind(2, date, 0)
        dbproc.dbbind(3, price, 0)
        dbproc.dbbind(4, priceavg, 0)
        dbproc.dbbind(5, sales)
        dbproc.dbbind(6, salesavg)
        dbproc.dbbind(7, sumsale)

        # dbaltbind() binds compute columns.
        dbproc.dbaltbind(1, 1, pcount)

        print "\nAccounts:"
        print "---------\n"
        print "%-5s  %-26s  %-6s  %-10s  %-5s  %-10s  %-10s\n" \
              % ("pubid", "date", "price", "avg(price)",
                 "sales", "avg(sales)", "sum(sales)")

        # Print out each result row, using different statements
        # depending on whether the row is a regular row or a compute
        # row.
        while 1:
            row_code = dbproc.dbnextrow()
            if row_code == NO_MORE_ROWS:
                break
            if row_code == REG_ROW:
                print "%5s  %26s  %6s  %10s  %5d  %10d  %10d" \
                      % (pubid, date, price, priceavg, sales.get(), 
                         salesavg.get(), sumsale.get())
            else:
                print "title count:  %d\n" % pcount.get()
dbexit()
